#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/i2c-dev.h>

#include "i2c.h"

int i2c_open(const struct driver *drv)
{
    assert(drv != NULL);

    struct i2c *i2c = (struct i2c *)(drv->dev);

    if((i2c->i2c_file = open(i2c->i2c_device, O_RDWR)) < 0) {
        fprintf(stderr, "Failed to open the i2c bus\r\n");
        return -1;
    }

    if(ioctl(i2c->i2c_file, I2C_SLAVE, i2c->slave_addr) < 0) {
        fprintf(stderr, "Failed to acquire bus access and/or talk to slave.\r\n");
        return -2;
    }

    return 0;
}

int i2c_close(const struct driver *drv)
{
    assert(drv != NULL);

    struct i2c *i2c = (struct i2c *)(drv->dev);

    close(i2c->i2c_file);

    return 0;
}

int i2c_read(const struct driver *drv, char *buffer, unsigned int len)
{
    assert(drv != NULL);

    struct i2c *i2c = (struct i2c *)(drv->dev);

    if(read(i2c->i2c_file, buffer, len) != len) {
        fprintf(stderr, "Failed to read from the i2c bus.\r\n");
        return -1;
    }
    return len;
}

int i2c_write(const struct driver *drv, const char *buffer, unsigned int len)
{
    assert(drv != NULL);

    struct i2c *i2c = (struct i2c *)(drv->dev);

    if(write(i2c->i2c_file, buffer, len) != len) {
        fprintf(stderr, "Failed to write to the i2c bus.\r\n");
        return -1;
    }
    return len;
}
